/***************************************************************************
 *   Copyright (C) 2009 by Marcin Nawrocki <mar.nawrocki@gmail.com>        *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 3 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA .        *
 ***************************************************************************/

#include "plasma-ydpdict.h"
#include <QGraphicsLinearLayout>
#include <Plasma/TextEdit>
#include <Plasma/ToolButton>
#include <Plasma/ComboBox>
#include <Plasma/TreeView>
#include <Plasma/CheckBox>
#include <Plasma/RadioButton>
#include <Plasma/ToolTipContent>
#include <Plasma/ToolTipManager>
#include <Plasma/Theme>
#include <Plasma/Containment>
#include <QStringListModel>
#include <QStringList>
#include <KTextEdit>
#include <QTextEdit>
#include <KLineEdit>
#include <QTreeView>
#include <QToolButton>
#include <QCheckBox>
#include <KComboBox>
#include <QComboBox>
#include <QRadioButton>
#include <KConfigDialog>
#include <KFileDialog>
#include <QProcess>

QStringList PlasmaYdpdict::m_aStringList[] = {QStringList(), QStringList(), QStringList(), QStringList()};
const QString PlasmaYdpdict::m_asDataFiles[] = {QString("dict100.dat"), QString("dict101.dat"), 
                                                QString("dict200.dat"), QString("dict201.dat")};
const QString PlasmaYdpdict::m_asIndexFiles[] = {QString("dict100.idx"), QString("dict101.idx"), 
                                                 QString("dict200.idx"), QString("dict201.idx")};
const QString PlasmaYdpdict::m_sCollinsDictionaryKey = "collinsDictionaryDirectory";
const QString PlasmaYdpdict::m_sCollinsSamplesKey = "collinsSamplesDirectory";
const QString PlasmaYdpdict::m_sCollinsPlayerKey = "collinsSamplePlayer";
const QString PlasmaYdpdict::m_sLangenscheidtDictionaryKey = "langenscheidtDictionaryDirectory";
const QString PlasmaYdpdict::m_sLangenscheidtSamplesKey = "langenscheidtSamplesDirectory";
const QString PlasmaYdpdict::m_sLangenscheidtPlayerKey = "langenscheidtSamplePlayer";
const QString PlasmaYdpdict::m_sTranslucentBackgrKey = "translucentBackground";
const QString PlasmaYdpdict::m_sSolidBackgrKey = "solidBackground";
const QString PlasmaYdpdict::m_sOpenRecentlyUsedDictKey = "openRecentlyUsedDictionary";
const QString PlasmaYdpdict::m_sRecentlyUsedDictKey = "recentlyUsedDictionary";
const QString PlasmaYdpdict::m_sInitialDictionaryKey = "initialDictionary";
const QString PlasmaYdpdict::m_sMaxPositionsInHistoryKey = "maxPositionsInHistory";
const QString PlasmaYdpdict::m_sStoreHistoryKey = "storeHistory";
const QString PlasmaYdpdict::m_sPanelBackgrColor = "panelBackgroundColor";
const QString PlasmaYdpdict::m_sTextColor = "textColor";
const QString PlasmaYdpdict::m_sIconName = "accessories-dictionary";
const QSizeF PlasmaYdpdict::m_appletMinSize = QSizeF(375, 200);
const int PlasmaYdpdict::m_nMaxHistoryItems = 32;
const int PlasmaYdpdict::m_nMaxPanelWidth = 150;
QProcess PlasmaYdpdict::m_process;
int PlasmaYdpdict::m_nTimeout = 2000;

PlasmaYdpdict::PlasmaYdpdict(QObject *parent, const QVariantList &args)
:Plasma::PopupApplet(parent, args), m_pYdpdict(NULL), 
m_pClipboard(QApplication::clipboard()), m_bSamplePlayed(false), m_bShowPopup(false),
m_bTrackClipboard(false), m_bHistoryUpdate(false), m_bShow(false), m_nMode(QClipboard::Clipboard), 
m_nCurrLang(None)
{
    m_pEnDeButton           = new Plasma::ToolButton;
    m_pPlButton             = new Plasma::ToolButton;
    m_pPlayButton           = new Plasma::ToolButton;
    m_pClipboardCheckBox    = new Plasma::CheckBox;
    m_pWindowCheckBox       = new Plasma::CheckBox;
    m_pSelfSelectCheckBox   = new Plasma::CheckBox;
    m_pComboBox             = new Plasma::ComboBox;
    m_pTextEdit             = new Plasma::TextEdit;
    m_pTreeView             = new Plasma::TreeView;
    m_pEnRadioButton        = new Plasma::RadioButton;
    m_pDeRadioButton        = new Plasma::RadioButton;
    m_pWordsStringListModel = new QStringListModel;
    m_pMainWidget           = new QGraphicsWidget(this);
    m_pMainLayout           = new QGraphicsLinearLayout(Qt::Vertical);
    m_pWordsLayout          = new QGraphicsLinearLayout(Qt::Vertical);
    m_pButtonLayout         = new QGraphicsLinearLayout(Qt::Horizontal);
    m_pDictLayout           = new QGraphicsLinearLayout(Qt::Horizontal);

    // buttons
    m_pPlButton->nativeWidget()->setIcon(QIcon(":/icons/pl.png"));
    m_pPlayButton->nativeWidget()->setIcon(KIcon("text-speak"));
    m_pPlButton->nativeWidget()->setToolTip(i18n("Switch to polish"));
    m_pPlayButton->nativeWidget()->setToolTip(i18n("Play sample"));
    m_pClipboardCheckBox->nativeWidget()->setIcon(KIcon("fifteenpuzzle"));
    m_pWindowCheckBox->nativeWidget()->setIcon(KIcon("apport"));
    m_pSelfSelectCheckBox->nativeWidget()->setIcon(KIcon("input-mouse"));
    m_pClipboardCheckBox->nativeWidget()->setToolTip(i18n("Track clipboard"));
    m_pWindowCheckBox->nativeWidget()->setToolTip(i18n("Auto show window"));
    m_pSelfSelectCheckBox->nativeWidget()->setToolTip(i18n("Track self selection"));
    m_pWindowCheckBox->setEnabled(m_bTrackClipboard);
    m_pEnDeButton->setAutoRaise(false);
    m_pPlButton->setAutoRaise(false);
    m_pPlayButton->setAutoRaise(false);
    m_pDeRadioButton->nativeWidget()->setToolTip(i18n("Switch to Langenscheidt"));
    m_pEnRadioButton->nativeWidget()->setToolTip(i18n("Switch to Collins"));
    m_pDeRadioButton->setImage(":/icons/de.png");
    m_pEnRadioButton->setImage(":/icons/en.png");
    m_pDeRadioButton->nativeWidget()->setAutoExclusive(false);
    m_pEnRadioButton->nativeWidget()->setAutoExclusive(false);
    m_pButtonLayout->addItem(m_pEnDeButton);
    m_pButtonLayout->addItem(m_pPlButton);
    m_pButtonLayout->addItem(m_pPlayButton);  
    m_pButtonLayout->addItem(m_pClipboardCheckBox);
    m_pButtonLayout->addItem(m_pWindowCheckBox);
    m_pButtonLayout->addItem(m_pSelfSelectCheckBox);
    m_pButtonLayout->addItem(m_pEnRadioButton);
    m_pButtonLayout->addItem(m_pDeRadioButton);
    m_pButtonLayout->setItemSpacing(2, 20);
    m_pButtonLayout->setItemSpacing(5, 20);
    m_pEnDeButton->setZValue(3);
    m_pPlButton->setZValue(3);
    m_pPlayButton->setZValue(3);
    m_pClipboardCheckBox->setZValue(3);
    m_pWindowCheckBox->setZValue(3);
    m_pSelfSelectCheckBox->setZValue(3);
    m_pDeRadioButton->setZValue(3);
    m_pEnRadioButton->setZValue(3);
    m_pButtonLayout->addStretch();
    
    // words
    m_pTreeView->nativeWidget()->setEditTriggers(QAbstractItemView::NoEditTriggers);
    m_pTreeView->nativeWidget()->setRootIsDecorated(false);
    m_pTreeView->nativeWidget()->setHeaderHidden(true);
    m_pTreeView->nativeWidget()->setMouseTracking(false);
    m_pTreeView->nativeWidget()->setHorizontalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
    m_pTreeView->nativeWidget()->setModel(m_pWordsStringListModel);
    m_pTreeView->setZValue(1);
    m_pComboBox->nativeWidget()->setEditable(true);
    m_pComboBox->nativeWidget()->setInsertPolicy(QComboBox::NoInsert);
    m_pComboBox->nativeWidget()->setSizeAdjustPolicy(QComboBox::AdjustToMinimumContentsLength);
    m_pComboBox->setZValue(2);
    m_pWordsLayout->addItem(m_pComboBox);
    m_pWordsLayout->addItem(m_pTreeView);
    m_pWordsLayout->setMaximumWidth(m_nMaxPanelWidth);

    // text
    m_pTextEdit->nativeWidget()->setReadOnly(true);
    m_pTextEdit->setZValue(1);
    m_pDictLayout->addItem(m_pWordsLayout);
    m_pDictLayout->addItem(m_pTextEdit);
    m_pDictLayout->setMinimumSize(m_appletMinSize);

    // main window
    m_pMainLayout->addItem(m_pButtonLayout);
    m_pMainLayout->addItem(m_pDictLayout);
    m_pMainWidget->setLayout(m_pMainLayout);

    connect(m_pTreeView->nativeWidget(), SIGNAL(clicked(const QModelIndex&)), SLOT(updateDescription()));
    connect(m_pTreeView->nativeWidget(), SIGNAL(doubleClicked(const QModelIndex&)), SLOT(playSound()));
    connect(m_pComboBox->nativeWidget()->lineEdit(), SIGNAL(textEdited(const QString&)), SLOT(wordEdited(const QString&)));
    connect(m_pComboBox->nativeWidget(), SIGNAL(currentIndexChanged (int)), SLOT(currentIndexChanged (int)));
    connect(m_pComboBox->nativeWidget(), SIGNAL(returnPressed(const QString&)), SLOT(returnPressed(const QString&)));
    connect(m_pEnDeButton->nativeWidget(), SIGNAL(clicked(bool)), SLOT(changeLanguage()));
    connect(m_pPlButton->nativeWidget(), SIGNAL(clicked(bool)), SLOT(changeLanguage()));
    connect(m_pPlayButton->nativeWidget(), SIGNAL(clicked(bool)), SLOT(playSound()));
    connect(m_pClipboardCheckBox->nativeWidget(), SIGNAL(stateChanged(int)), SLOT(clipboardChecked(int)));
    connect(m_pWindowCheckBox->nativeWidget(), SIGNAL(stateChanged(int)), SLOT(windowChecked(int)));
    connect(&m_process, SIGNAL(finished(int, QProcess::ExitStatus)), SLOT(finished()));
    connect(m_pClipboard, SIGNAL(changed(QClipboard::Mode)), SLOT(clipboardModeChanged(QClipboard::Mode)));
    connect(m_pClipboard, SIGNAL(dataChanged()), SLOT(clipboardChanged()));
    connect(m_pClipboard, SIGNAL(selectionChanged()), SLOT(selectionChanged()));   
    connect(m_pEnRadioButton->nativeWidget(), SIGNAL(clicked(bool)), SLOT(englishClicked(bool)));
    connect(m_pDeRadioButton->nativeWidget(), SIGNAL(clicked(bool)), SLOT(germanClicked(bool)));
    connect(Plasma::Theme::defaultTheme(), SIGNAL(themeChanged()), SLOT(initialize()));
    connect(&m_timer, SIGNAL(timeout()), SLOT(timeout()));
    
    setAspectRatioMode(Plasma::IgnoreAspectRatio);
    setPopupIcon(m_sIconName);
    m_timer.setSingleShot(true);
}

PlasmaYdpdict::~PlasmaYdpdict()
{
    storeHistory(); /* in case of logout */
    ydpdict_close(m_pYdpdict);
    delete graphicsWidget();
}

void PlasmaYdpdict::fillQStringList(ydpdict_t* ydpdict, QStringList *pStringList)
{
    pStringList->clear();
    const int count = ydpdict_get_count(ydpdict);
    for (int i=0; i<count; i++)
        pStringList->append(QString::fromUtf8(ydpdict_get_word(ydpdict, i)));
}

void PlasmaYdpdict::initialize()
{
    Plasma::Containment *pContainment = Plasma::Applet::containment();
    if (pContainment->containmentType() != Plasma::Containment::PanelContainment)
        setBackgroundHints(globalConfig().readEntry(m_sTranslucentBackgrKey) == "true" ? TranslucentBackground : DefaultBackground);
    bool bSolidBackgr = globalConfig().readEntry(m_sSolidBackgrKey) == "true" ? true : false;
    m_pTreeView->nativeWidget()->viewport()->setAutoFillBackground(bSolidBackgr);
    m_pTextEdit->nativeWidget()->viewport()->setAutoFillBackground(bSolidBackgr);
    m_pComboBox->nativeWidget()->lineEdit()->setAutoFillBackground(bSolidBackgr);
    m_pComboBox->nativeWidget()->view()->viewport()->setAutoFillBackground(true);
    if (bSolidBackgr)
    {
        QPalette palette;
        palette.setColor(QPalette::Base, QColor(globalConfig().readEntry(m_sPanelBackgrColor)));
        m_pTreeView->nativeWidget()->viewport()->setPalette(palette);
        m_pTextEdit->nativeWidget()->viewport()->setPalette(palette);
        m_pComboBox->setPalette(palette);
        m_pComboBox->nativeWidget()->view()->viewport()->setPalette(palette);
        palette.setColor(QPalette::Text, QColor(globalConfig().readEntry(m_sTextColor)));
        m_pTreeView->nativeWidget()->setPalette(palette);
        m_pTextEdit->nativeWidget()->setPalette(palette);
        m_pComboBox->nativeWidget()->setPalette(palette);
        m_pComboBox->nativeWidget()->view()->setPalette(palette);
    }
    else
    {
        QPalette palette;
        palette.setColor(QPalette::Text, Plasma::Theme::defaultTheme()->color(Plasma::Theme::TextColor));
        m_pTreeView->nativeWidget()->setPalette(palette);
        m_pTextEdit->nativeWidget()->setPalette(palette);
        m_pComboBox->nativeWidget()->setPalette(palette);
        palette.setColor(QPalette::Base, Plasma::Theme::defaultTheme()->color(Plasma::Theme::BackgroundColor));
        m_pComboBox->nativeWidget()->view()->setPalette(palette);
        m_pComboBox->nativeWidget()->view()->viewport()->setPalette(palette);
    }
    
    ydpdict_t* temp = NULL;
    bool english = true, german = true;
    ydpdict_close(m_pYdpdict);
    english = ((temp = openDatabase(EnPl)) == NULL ? false : english);
    if (english)
        fillQStringList(temp, &m_aStringList[EnPl]);
    ydpdict_close(temp);
    english = ((temp = openDatabase(PlEn)) == NULL ? false : english);
    if (english)
        fillQStringList(temp, &m_aStringList[PlEn]);
    ydpdict_close(temp);
    german = ((temp = openDatabase(DePl)) == NULL ? false : german);
    if (german)
        fillQStringList(temp, &m_aStringList[DePl]);
    ydpdict_close(temp);
    german = ((temp = openDatabase(PlDe)) == NULL ? false : german);
    if (german)
        fillQStringList(temp, &m_aStringList[PlDe]);
    ydpdict_close(temp);
    int nDictToUse;
    if (globalConfig().readEntry(m_sOpenRecentlyUsedDictKey, "true") == "true")
	nDictToUse = globalConfig().readEntry(m_sRecentlyUsedDictKey, "0").toInt();
    else
	nDictToUse = globalConfig().readEntry(m_sInitialDictionaryKey, "0").toInt();
    if (english && german)
    {
        m_nCurrLang = (CurrentLanguage)nDictToUse;
    }
    else if (english || german)
    {
        if (german)
	    m_nCurrLang = (nDictToUse >= DePl && nDictToUse <= PlDe) ? (CurrentLanguage)nDictToUse : DePl;
	else
	    m_nCurrLang = (nDictToUse >= EnPl && nDictToUse <= PlEn) ? (CurrentLanguage)nDictToUse : EnPl;
    }
    else
        m_nCurrLang = None;
    m_pEnRadioButton->setChecked(m_nCurrLang >= EnPl && m_nCurrLang <= PlEn);
    m_pDeRadioButton->setChecked(m_nCurrLang >= DePl && m_nCurrLang <= PlDe);
    m_pEnRadioButton->setEnabled(english && german);
    m_pDeRadioButton->setEnabled(english && german);
    m_pComboBox->nativeWidget()->setCurrentIndex(0);
    for (int i=0; i<m_pComboBox->nativeWidget()->count(); i++)
    {
        int nLang = m_pComboBox->nativeWidget()->itemData(i).toInt();
        if ((!english && nLang >= EnPl && nLang <= PlEn) ||
            (!german && nLang >= DePl && nLang <= PlDe))
	{
	    m_pComboBox->nativeWidget()->removeItem(i);
	    i--;
	}
    }
    int maxCount = globalConfig().readEntry(m_sMaxPositionsInHistoryKey, QString(m_nMaxHistoryItems)).toInt();;
    m_pComboBox->nativeWidget()->setMaxCount(maxCount);
    updateWidgets();
}

void PlasmaYdpdict::init()
{
    initialize();
    readHistory();
}

void PlasmaYdpdict::destroy()
{
    storeHistory(); /* in case of applet remove */
    Plasma::PopupApplet::destroy();
}

QGraphicsWidget* PlasmaYdpdict::graphicsWidget()
{
    return m_pMainWidget;
}

ydpdict_t* PlasmaYdpdict::openDatabase(int lang)
{
    if (lang == None)
        return NULL;
    QString dictDir = globalConfig().readEntry(lang <= PlEn ? m_sCollinsDictionaryKey : m_sLangenscheidtDictionaryKey)+"/";
    QString dataFile = dictDir+PlasmaYdpdict::m_asDataFiles[lang];
    QString indexFile = dictDir+PlasmaYdpdict::m_asIndexFiles[lang];
    ydpdict_t* pYdpdict = ydpdict_open(dataFile.toLatin1(), indexFile.toLatin1(), YDPDICT_ENCODING_UTF8);
    if (!pYdpdict)
    {
        dataFile = dictDir+PlasmaYdpdict::m_asDataFiles[lang].toUpper();
        indexFile = dictDir+PlasmaYdpdict::m_asIndexFiles[lang].toUpper();
        pYdpdict = ydpdict_open(dataFile.toLatin1(), indexFile.toLatin1(), YDPDICT_ENCODING_UTF8);
    }
    return pYdpdict;
}

void PlasmaYdpdict::updateWidgets()
{
    m_pEnDeButton->nativeWidget()->setIcon(QIcon(m_nCurrLang <= PlEn ? ":/icons/en.png" : ":/icons/de.png"));
    m_pEnDeButton->nativeWidget()->setToolTip(m_nCurrLang <= PlEn ? i18n("Switch to english") : i18n("Switch to deutch"));
    m_pYdpdict = openDatabase(m_nCurrLang);
    if (m_pYdpdict)
    {
	m_pWordsStringListModel->setStringList(m_aStringList[m_nCurrLang]);
        QString word = m_pComboBox->nativeWidget()->lineEdit()->text();
        int index = findWord(word);
        m_pTreeView->nativeWidget()->setCurrentIndex(m_pTreeView->nativeWidget()->model()->index(index, 0));
        m_pTextEdit->nativeWidget()->setText(QString::fromUtf8(ydpdict_read_xhtml(m_pYdpdict, index)));
        Plasma::ToolTipContent toolTipData;
        toolTipData.setMainText(m_nCurrLang <= PlEn ? "Collins" : "Langenscheidt");
        toolTipData.setSubText(m_nCurrLang <= PlEn ? 
                              (m_nCurrLang == PlEn ? i18n("polish -> english") : i18n("english -> polish")) : 
                              (m_nCurrLang == PlDe ? i18n("polish -> german") : i18n("german -> polish")));
        toolTipData.setImage(m_nCurrLang <= PlEn ? QIcon(":/icons/en.png") : QIcon(":/icons/de.png"));
        Plasma::ToolTipManager::self()->setContent(this, toolTipData);
        m_pComboBox->nativeWidget()->lineEdit()->selectAll();    
	globalConfig().writeEntry(m_sRecentlyUsedDictKey, (int)m_nCurrLang);
        m_pEnRadioButton->setChecked(m_nCurrLang >= EnPl && m_nCurrLang <= PlEn);
        m_pDeRadioButton->setChecked(m_nCurrLang >= DePl && m_nCurrLang <= PlDe);
    }
    else
    {
	m_pWordsStringListModel->setStringList(QStringList());
        m_pTextEdit->nativeWidget()->clear();
        Plasma::ToolTipManager::self()->clearContent(this);
        m_pComboBox->nativeWidget()->lineEdit()->clear();
    }
    enableWidgets(m_pYdpdict);
    enablePlayButton(true);
}

void PlasmaYdpdict::wordEdited(const QString &text, bool startTimer)
{
    int index = ydpdict_find_word(m_pYdpdict, text.toUtf8());
    if (index != -1)
    {
        m_pTreeView->nativeWidget()->setCurrentIndex(m_pTreeView->nativeWidget()->model()->index(index, 0));
        m_pTextEdit->nativeWidget()->setText(QString::fromUtf8(ydpdict_read_xhtml(m_pYdpdict, index)));
        enablePlayButton(true);
    }
    if (startTimer)
        m_timer.start(m_nTimeout);
}

void PlasmaYdpdict::updateDescription()
{
    int index = m_pTreeView->nativeWidget()->currentIndex().row();
    m_pTextEdit->nativeWidget()->setText(QString::fromUtf8(ydpdict_read_xhtml(m_pYdpdict, index)));
    QString text = QString::fromUtf8(ydpdict_get_word(m_pYdpdict, index));
    m_pComboBox->nativeWidget()->lineEdit()->setText(text);
    insertHistoryItem(text);
    enablePlayButton(true);
}

void PlasmaYdpdict::changeLanguage()
{
    switch (m_nCurrLang)
    {
        case PlEn:
            m_nCurrLang = EnPl;
            break;
        case EnPl:
            m_nCurrLang = PlEn;
            break;
        case PlDe:
            m_nCurrLang = DePl;
            break;
        case DePl:
            m_nCurrLang = PlDe;
            break;
        default:
            break;
    }
    updateWidgets();
    m_pComboBox->nativeWidget()->lineEdit()->setFocus();
}

void PlasmaYdpdict::enableWidgets(bool enable)
{
    m_pTextEdit->nativeWidget()->setEnabled(enable);
    m_pComboBox->setEnabled(enable);
    m_pTreeView->nativeWidget()->setEnabled(enable);
    m_pPlButton->setEnabled(enable ? (m_nCurrLang == EnPl || m_nCurrLang == DePl) : false);
    m_pEnDeButton->setEnabled(enable ? (m_nCurrLang == PlEn || m_nCurrLang == PlDe) : false);
    enablePlayButton(enable);
    m_bShowPopup = (enable ? m_bShowPopup : false);
    m_bTrackClipboard = (enable ? m_bTrackClipboard : false);
    m_pWindowCheckBox->setChecked(m_bShowPopup);
    m_pClipboardCheckBox->setChecked(m_bTrackClipboard);
    m_pSelfSelectCheckBox->setChecked(enable ? m_pSelfSelectCheckBox->isChecked() : false);
    m_pWindowCheckBox->setEnabled(enable && m_bTrackClipboard && m_bShow);
    m_pClipboardCheckBox->setEnabled(enable);
    m_pSelfSelectCheckBox->setEnabled(enable ? !m_bTrackClipboard : false);
}

void PlasmaYdpdict::createConfigurationInterface(KConfigDialog *parent)
{
    QWidget *pWidgetConfig = new QWidget;
    m_CollinsForm.setupUi(pWidgetConfig);
    m_CollinsForm.kurlreqDictionary->fileDialog()->setMode(KFile::Directory);
    m_CollinsForm.kurlreqDictionary->lineEdit()->setText(globalConfig().readEntry(m_sCollinsDictionaryKey));
    m_CollinsForm.kurlreqSamples->fileDialog()->setMode(KFile::Directory);
    m_CollinsForm.kurlreqSamples->lineEdit()->setText(globalConfig().readEntry(m_sCollinsSamplesKey));
    m_CollinsForm.kurlreqPlayer->lineEdit()->setText(globalConfig().readEntry(m_sCollinsPlayerKey));
    parent->addPage(pWidgetConfig, "Collins", ":/icons/flagEn.png");
    pWidgetConfig = new QWidget;
    m_LangenscheidtForm.setupUi(pWidgetConfig);
    m_LangenscheidtForm.kurlreqDictionary->fileDialog()->setMode(KFile::Directory);
    m_LangenscheidtForm.kurlreqDictionary->lineEdit()->setText(globalConfig().readEntry(m_sLangenscheidtDictionaryKey));
    m_LangenscheidtForm.kurlreqSamples->fileDialog()->setMode(KFile::Directory);
    m_LangenscheidtForm.kurlreqSamples->lineEdit()->setText(globalConfig().readEntry(m_sLangenscheidtSamplesKey));
    m_LangenscheidtForm.kurlreqPlayer->lineEdit()->setText(globalConfig().readEntry(m_sLangenscheidtPlayerKey));
    parent->addPage(pWidgetConfig, "Langenscheidt", ":/icons/flagDe.png");
    pWidgetConfig = new QWidget;
    m_AppearanceForm.setupUi(pWidgetConfig);
    Plasma::Containment *pContainment = Plasma::Applet::containment();
    m_AppearanceForm.translBackgrCheckBox->setEnabled(pContainment->containmentType() == Plasma::Containment::DesktopContainment);
    m_AppearanceForm.translBackgrCheckBox->setChecked(globalConfig().readEntry(m_sTranslucentBackgrKey) == "true" ? true : false);
    bool bSolidBackgr = globalConfig().readEntry(m_sSolidBackgrKey) == "true" ? true : false;
    m_AppearanceForm.solidBackgrCheckBox->setChecked(bSolidBackgr);
    m_AppearanceForm.textColorTextLabel->setEnabled(bSolidBackgr);
    m_AppearanceForm.panelColorTextLabel->setEnabled(bSolidBackgr);
    m_AppearanceForm.textColorButton->setEnabled(bSolidBackgr);
    m_AppearanceForm.panelColorButton->setEnabled(bSolidBackgr);
    m_AppearanceForm.textColorButton->setColor(globalConfig().readEntry(m_sTextColor));
    m_AppearanceForm.panelColorButton->setColor(globalConfig().readEntry(m_sPanelBackgrColor));
    parent->addPage(pWidgetConfig, i18n("Appearance"), "format-fill-color");
    pWidgetConfig = new QWidget;
    m_GeneralForm.setupUi(pWidgetConfig);
    bool bOpenRecentlyUsedDict = globalConfig().readEntry(m_sOpenRecentlyUsedDictKey, "true") == "true";
    m_GeneralForm.openRecentlyUsedDictRadioButton->setChecked(bOpenRecentlyUsedDict);
    m_GeneralForm.openSelectedDictRadioButton->setChecked(!bOpenRecentlyUsedDict);
    m_GeneralForm.initialDictComboBox->addItem(i18n("Collins, English"));
    m_GeneralForm.initialDictComboBox->addItem(i18n("Collins, Polish"));
    m_GeneralForm.initialDictComboBox->addItem(i18n("Langenscheidt, Deutsch"));
    m_GeneralForm.initialDictComboBox->addItem(i18n("Langenscheidt, Polish"));
    m_GeneralForm.initialDictComboBox->setCurrentIndex(globalConfig().readEntry(m_sInitialDictionaryKey, "0").toInt());
    m_GeneralForm.maxPosInHistorySpinBox->setMaximum(m_nMaxHistoryItems);
    m_GeneralForm.maxPosInHistorySpinBox->setValue(globalConfig().readEntry(m_sMaxPositionsInHistoryKey, QString(m_nMaxHistoryItems)).toInt());
    m_GeneralForm.storeHistoryCheckBox->setChecked(globalConfig().readEntry(m_sStoreHistoryKey) == "true" ? true : false);
    parent->addPage(pWidgetConfig, i18n("General"), "configure");
    connect(parent, SIGNAL(okClicked()), this, SLOT(configAccepted()));
}

void PlasmaYdpdict::configAccepted()
{
    globalConfig().writeEntry(m_sCollinsDictionaryKey, m_CollinsForm.kurlreqDictionary->lineEdit()->text());
    globalConfig().writeEntry(m_sCollinsSamplesKey, m_CollinsForm.kurlreqSamples->lineEdit()->text());
    globalConfig().writeEntry(m_sCollinsPlayerKey, m_CollinsForm.kurlreqPlayer->lineEdit()->text());
    globalConfig().writeEntry(m_sLangenscheidtDictionaryKey, m_LangenscheidtForm.kurlreqDictionary->lineEdit()->text());
    globalConfig().writeEntry(m_sLangenscheidtSamplesKey, m_LangenscheidtForm.kurlreqSamples->lineEdit()->text());
    globalConfig().writeEntry(m_sLangenscheidtPlayerKey, m_LangenscheidtForm.kurlreqPlayer->lineEdit()->text());
    globalConfig().writeEntry(m_sTranslucentBackgrKey, m_AppearanceForm.translBackgrCheckBox->isChecked() ? "true" : "false");
    globalConfig().writeEntry(m_sSolidBackgrKey, m_AppearanceForm.solidBackgrCheckBox->isChecked() ? "true" : "false");
    globalConfig().writeEntry(m_sPanelBackgrColor, m_AppearanceForm.panelColorButton->color().name());
    globalConfig().writeEntry(m_sTextColor, m_AppearanceForm.textColorButton->color().name());
    globalConfig().writeEntry(m_sOpenRecentlyUsedDictKey, m_GeneralForm.openRecentlyUsedDictRadioButton->isChecked());
    globalConfig().writeEntry(m_sInitialDictionaryKey, m_GeneralForm.initialDictComboBox->currentIndex());
    globalConfig().writeEntry(m_sMaxPositionsInHistoryKey, m_GeneralForm.maxPosInHistorySpinBox->value());
    globalConfig().writeEntry(m_sStoreHistoryKey, m_GeneralForm.storeHistoryCheckBox->isChecked() ? "true" : "false");
    initialize();
}

void PlasmaYdpdict::playSound()
{
    QString program;
    if (m_nCurrLang == EnPl)
        program = globalConfig().readEntry(m_sCollinsPlayerKey);
    else
    if (m_nCurrLang == DePl)
        program = globalConfig().readEntry(m_sLangenscheidtPlayerKey);
    if (m_nCurrLang == EnPl || m_nCurrLang == DePl)
    {
        m_process.start(program, QStringList(getPathToSample()));
        m_bSamplePlayed = true;
    }
    enablePlayButton(!m_bSamplePlayed);
}

void PlasmaYdpdict::finished()
{
    m_bSamplePlayed = false;
    enablePlayButton(true);
}

QString PlasmaYdpdict::getPathToSample()
{
    int index = m_pTreeView->nativeWidget()->currentIndex().row();
    QString path = globalConfig().readEntry(m_nCurrLang <= PlEn ? m_sCollinsSamplesKey : m_sLangenscheidtSamplesKey);
    QString format = (m_nCurrLang <= PlEn ? "/S%.3d/%.6d.WAV" : "/s%.3d/%.6d.wav");
    path += QString().sprintf(format.toLatin1(), (index+1)/1000, index+1);
    return path;
}

void PlasmaYdpdict::enablePlayButton(bool enable)
{
    bool enabled = enable && !m_bSamplePlayed && QFile().exists(getPathToSample());
    if (m_nCurrLang == EnPl)
        enabled &= QFile().exists(globalConfig().readEntry(m_sCollinsPlayerKey));
    else
    if (m_nCurrLang == DePl)
        enabled &= QFile().exists(globalConfig().readEntry(m_sLangenscheidtPlayerKey));
    else
        enabled = false;
    m_pPlayButton->setEnabled(enabled);
}

void PlasmaYdpdict::popupEvent(bool show)
{
    m_bShow = show;
    if (show)
    {
        m_pComboBox->nativeWidget()->lineEdit()->grabKeyboard();
        m_pComboBox->nativeWidget()->lineEdit()->selectAll();
        m_pWindowCheckBox->setEnabled(m_pYdpdict && m_bTrackClipboard);
    }
    else
    {
        m_pComboBox->nativeWidget()->lineEdit()->releaseKeyboard();
        m_pWindowCheckBox->setEnabled(false);
    }
}

void PlasmaYdpdict::clipboardModeChanged(QClipboard::Mode mode)
{
    m_nMode = mode;
}

void PlasmaYdpdict::clipboardChanged()
{
    if (m_bTrackClipboard && m_nMode == QClipboard::Clipboard)
	handleClipboardChange(QClipboard::Clipboard);
}

void PlasmaYdpdict::selectionChanged()
{
    if (m_bTrackClipboard || 
       (m_pSelfSelectCheckBox->isChecked() && 
        m_pTextEdit->nativeWidget()->textCursor().hasSelection()))
	handleClipboardChange(QClipboard::Selection);
}

void PlasmaYdpdict::handleClipboardChange(QClipboard::Mode mode)
{
    if (m_timer.isActive())
	m_timer.stop();
    QString word = m_pClipboard->text(mode);
    m_pComboBox->nativeWidget()->lineEdit()->setText(word);
    insertHistoryItem(word);
    findWord(word);
    wordEdited(word, false);
    if (m_bShowPopup)
	showPopup();
}

int PlasmaYdpdict::findWord(QString& word)
{
    int index;
    while ((index = ydpdict_find_word(m_pYdpdict, word.toUtf8())) == -1)
        word.chop(1);
    return index;
}

void PlasmaYdpdict::clipboardChecked(int state)
{
    m_bTrackClipboard = state;
    m_pWindowCheckBox->setEnabled(m_bTrackClipboard && m_bShow);
    m_pSelfSelectCheckBox->setEnabled(!state);
}

void PlasmaYdpdict::windowChecked(int state)
{
    m_bShowPopup = state;
}

void PlasmaYdpdict::currentIndexChanged(int index)
{
    if (m_timer.isActive())
        m_timer.stop();
    if (!m_bHistoryUpdate)
    {
        int nLang = m_pComboBox->nativeWidget()->itemData(index).toInt();
        if (m_nCurrLang != nLang)
	{
	    m_nCurrLang = (CurrentLanguage)nLang;
	    updateWidgets();
	}
        wordEdited(m_pComboBox->nativeWidget()->itemText(index), false);
    }
}

void PlasmaYdpdict::germanClicked(bool enabled)
{
    if (enabled)
        m_pEnRadioButton->setChecked(false);
    else
        m_pDeRadioButton->setChecked(true);
    if (m_nCurrLang <= PlEn)
    {
        m_nCurrLang = DePl;
        updateWidgets();
    }
}

void PlasmaYdpdict::englishClicked(bool enabled)
{
    if (enabled)
        m_pDeRadioButton->setChecked(false);
    else
        m_pEnRadioButton->setChecked(true);
    if (m_nCurrLang >= DePl)
    {
        m_nCurrLang = EnPl;
        updateWidgets();
    }
}

QIcon PlasmaYdpdict::getIcon(CurrentLanguage lang)
{
    switch (lang)
    {
      case EnPl:
	return QIcon(":/icons/en-pl.png");
      case PlEn:
	return QIcon(":/icons/pl-en.png");
      case DePl:
	return QIcon(":/icons/de-pl.png");
      case PlDe:
	return QIcon(":/icons/pl-de.png");
      default:
	return QIcon();
    }
}

void PlasmaYdpdict::insertHistoryItem(const QString &word)
{
    m_bHistoryUpdate = true;
    m_pComboBox->nativeWidget()->insertItem(0, getIcon(m_nCurrLang), word, QVariant(m_nCurrLang));   
    m_pComboBox->nativeWidget()->setCurrentIndex(0);
    m_bHistoryUpdate = false;
}

void PlasmaYdpdict::timeout()
{
    QString text = m_pComboBox->nativeWidget()->lineEdit()->text();
    if (!text.isEmpty())
        insertHistoryItem(text);
}

void PlasmaYdpdict::returnPressed(const QString& text)
{
    if (m_timer.isActive())
    {
        m_timer.stop();
        insertHistoryItem(text);
    }
}

void PlasmaYdpdict::storeHistory()
{
    for (int i=0; i<m_nMaxHistoryItems; i++)
    {
	QString key = QString().sprintf("item%02d", i);
	if (globalConfig().hasKey(key))
	    globalConfig().deleteEntry(key);
    }
    if (globalConfig().readEntry(m_sStoreHistoryKey) == "true")
    {
	int count = m_pComboBox->nativeWidget()->count();
        for (int i=0; i<count; i++)
	{
	    QString value = m_pComboBox->nativeWidget()->itemData(count-i-1).toString()+m_pComboBox->nativeWidget()->itemText(count-i-1);
	    globalConfig().writeEntry(QString().sprintf("item%02d", i), value);
	}
    }
}

void PlasmaYdpdict::readHistory()
{
    int maxCount = globalConfig().readEntry(m_sMaxPositionsInHistoryKey, QString(m_nMaxHistoryItems)).toInt();
    if (globalConfig().readEntry(m_sStoreHistoryKey) == "true")
    {
	for (int i=0; i<maxCount; i++)
	{
	    QString key = QString().sprintf("item%02d", i);
	    if (globalConfig().hasKey(key))
	    {
		QString value = globalConfig().readEntry(key);
		CurrentLanguage lang = (CurrentLanguage)value.left(1).toInt();
		m_pComboBox->nativeWidget()->insertItem(0, getIcon(lang), value.right(value.size()-1), QVariant(lang));
	    }
	}
	m_pComboBox->nativeWidget()->setCurrentIndex(0);
    }
}

#include "build/plasma-ydpdict.moc"
