/***************************************************************************
 *   Copyright (C) 2009 by Marcin Nawrocki <mar.nawrocki@gmail.com>        *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 3 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA .        *
 ***************************************************************************/

#ifndef Ydpdict_HEADER
#define Ydpdict_HEADER

#include <Plasma/PopupApplet>
#include <QClipboard>
#include <QTimer>
#include "ui_dictionaryForm.h"
#include "ui_appearanceForm.h"
#include "ui_generalForm.h"
extern "C" { 
#include "ydpdict/ydpdict.h"
}

#define NO_OF_LISTS 4

class QGraphicsWidget;
class QGraphicsLinearLayout;
class QStringListModel;
class QStringList;
class QProcess;

namespace Plasma
{
    class ToolButton;
    class ComboBox;
    class TreeView;
    class TextEdit;
    class CheckBox;
    class RadioButton;
}

typedef enum
{ 
    None=-1, 
    EnPl, 
    PlEn, 
    DePl, 
    PlDe 
} CurrentLanguage;

class PlasmaYdpdict : public Plasma::PopupApplet
{
    Q_OBJECT
    public:
        PlasmaYdpdict(QObject *parent, const QVariantList &args);
        ~PlasmaYdpdict();
        QGraphicsWidget *graphicsWidget();
    public slots:
        void updateDescription();
        void wordEdited(const QString&, bool startTimer = true);
        void changeLanguage();
        void configAccepted();
        void playSound();
        void enablePlayButton(bool);
        void clipboardChanged();
	void selectionChanged();
        void clipboardModeChanged(QClipboard::Mode);
        void clipboardChecked(int);
        void windowChecked(int);
        void currentIndexChanged (int);
        void germanClicked(bool);
        void englishClicked(bool);
        void finished();
        void timeout();
        void returnPressed(const QString&);
    private:
        Ui_DictionaryForm m_CollinsForm;
        Ui_DictionaryForm m_LangenscheidtForm;
        Ui_AppearanceForm m_AppearanceForm;
        Ui_GeneralForm m_GeneralForm;
        ydpdict_t *m_pYdpdict;
        QClipboard *m_pClipboard;
        Plasma::ToolButton *m_pEnDeButton;
        Plasma::ToolButton *m_pPlButton;
        Plasma::ToolButton *m_pPlayButton;
        Plasma::CheckBox *m_pClipboardCheckBox;
        Plasma::CheckBox *m_pWindowCheckBox;
        Plasma::CheckBox *m_pSelfSelectCheckBox;
        Plasma::ComboBox *m_pComboBox;
        Plasma::RadioButton *m_pEnRadioButton;
        Plasma::RadioButton *m_pDeRadioButton;
        Plasma::TreeView *m_pTreeView;
        Plasma::TextEdit *m_pTextEdit;
        QGraphicsWidget *m_pMainWidget;
        QGraphicsLinearLayout *m_pMainLayout;
        QGraphicsLinearLayout *m_pButtonLayout;
        QGraphicsLinearLayout *m_pWordsLayout;
        QGraphicsLinearLayout *m_pDictLayout;
        QStringListModel *m_pWordsStringListModel;
	static QStringList m_aStringList[NO_OF_LISTS];
        QTimer m_timer;
        bool m_bSamplePlayed;
        bool m_bShowPopup;
        bool m_bTrackClipboard;
        bool m_bHistoryUpdate;
        bool m_bShow;
        QClipboard::Mode m_nMode;

        static const QString m_asDataFiles[];
        static const QString m_asIndexFiles[];
        static const int m_nMaxPanelWidth;
        static const QString m_sCollinsDictionaryKey;
        static const QString m_sCollinsSamplesKey;
        static const QString m_sCollinsPlayerKey;
        static const QString m_sLangenscheidtDictionaryKey;
        static const QString m_sLangenscheidtSamplesKey;
        static const QString m_sLangenscheidtPlayerKey;
        static const QString m_sTranslucentBackgrKey;
	static const QString m_sOpenRecentlyUsedDictKey;
	static const QString m_sRecentlyUsedDictKey;
	static const QString m_sInitialDictionaryKey;
	static const QString m_sMaxPositionsInHistoryKey;
	static const QString m_sStoreHistoryKey;
        static const QString m_sSolidBackgrKey;
        static const QString m_sPanelBackgrColor;
        static const QString m_sTextColor;
        static const QString m_sIconName;
        static const QSizeF m_appletMinSize;
        static QProcess m_process;
        static int m_nTimeout;
	static const int m_nMaxHistoryItems;
	CurrentLanguage m_nCurrLang;

        QString getPathToSample();
        void updateWidgets();
        void enableWidgets(bool);
	void initialize();
        void init();
	void destroy();
        void popupEvent(bool);
        ydpdict_t* openDatabase(int);
        void insertHistoryItem(const QString&);
        int findWord(QString&);
	void handleClipboardChange(QClipboard::Mode);
	void fillQStringList(ydpdict_t*, QStringList*);
	void storeHistory();
	void readHistory();
	QIcon getIcon(CurrentLanguage);
    protected:
        void createConfigurationInterface(KConfigDialog*);
};

K_EXPORT_PLASMA_APPLET(ydpdict, PlasmaYdpdict)
#endif
